/* 
 * $Id: LocalSAXBuilder.java,v 1.12 2005/08/16 14:37:11 buchmand Exp $
 *
 *********************************************************
 * Copyright (C) 2004 Dominik Jungo, David Buchmann      *
 *                                                       *
 * VeriNec Project                                       *
 *                                                       *
 * telecommunications ##### n  research group            *
 *           networks   #      university of fribourg    *
 *           security   #   s  department of informatics *
 *                                                       *
 * Dr. Ulrich Ultes-Nitsche, uun@unifr.ch                *
 * Dominik Jungo, dominik.jungo@unifr.ch                 *
 * David Buchmann, david.buchmann@unifr.ch               *
 *                                                       *
 * http://diuf.unifr.ch/tns/                             *
 *                                                       *
 * This program is copyrighted under the GNU General     *
 * Public License. Please see the file LICENSE.txt for   *
 * the details of that license.                          *
 *********************************************************
 */
package verinec.util;
 
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.logging.Logger;

import nT.util.ResourceUtility;

import org.jdom.input.SAXBuilder;

import verinec.VerinecException;

/** A SAXBuilder using local copies of the Verinec schemas.
 *  The Builder is set to be validating against schemas and sets the parser to 
 *  use all local .xsd files in /res/schemas/ within the classpath.
 *  Use only relative paths in xs:include statements, or you will still need
 *  network access.
 * <p>
 *  Because of a bug in Java 1.4 and 1.5 {@link #urlEncode(String) see urlEncode}, 
 *  this does only work if the path to the jar contains only us-ascii characters (spaces are ok).
 * </p>  
 * 
 * <p>The Class is uses a Singleton design pattern to guarantee a single instance 
 * of the Builder. The use singleton is reasonable because finding all schemas 
 * takes a long time and the build methods are reentrant.</p>
 *  
 * @author Dominik Jungo
 * @author David Buchmann
 * @version $Revision: 1.12 $ $Date: 2005/08/16 14:37:11 $
 */
public class LocalSAXBuilder extends SAXBuilder {
    private static LocalSAXBuilder instance;
    private Logger logger = Logger.getLogger(getClass().getName());
    
    /** This is a singleton, use {@link #instance()} */
    private LocalSAXBuilder() {
        //force org.jdom.SAXBuilder to use xerces and no other implementation, as schema validation depends on implementation
        super("org.apache.xerces.parsers.SAXParser");
        
        setFeature ("http://xml.org/sax/features/validation", true); //report validation error. document should specify schema (or dtd) 
        setValidation(true); //tell it jdom too. don't know if necessary or redundant
        setFeature ("http://apache.org/xml/features/validation/schema", true); //report schema validation errors
        setFeature ("http://apache.org/xml/features/validation/schema-full-checking",true); //full checking with grammar and everything
			
        try {
            String schemas = buildSchemaLocations();

            setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", schemas);
            //if we had documents without namespace, we would also set external-noNamespaceSchemaLocation to the schema for those documents.
        } catch (Throwable t) {
            logger.warning("Exception while trying to set local schema location. Will need network access to validate documents.");
            logger.throwing(getClass().getName(), "LocalSAXBuilder", t);
        }
        
    }

    
    /** Creates an instance of LocalSAXBuilder if it does not exist or returns the existing one otherwise.
     * 
     * @return an instance of LocalSAXBuilder 
     */ 
    public static LocalSAXBuilder instance() {
        if(instance==null){
            instance = new LocalSAXBuilder();
        }
        return instance;
    }
    
    /** Collect the string for the schemaLocation attribute.
     * It has the format {namespace uri}*
     * Because of a bug in Java 1.4 and 1.5 {@link #urlEncode(String) see urlEncode}, 
     * this does only work if the path to the jar contains no non-us-ascii characters.
     * @return A schema location string appropriate for the xerces property.
     * @throws VerinecException If we have a problem with the classpath.
     */
    private String buildSchemaLocations() throws VerinecException{
        String result = "";
        Iterator iter;
        try {
            iter = ResourceUtility.getSystemResources("/res/schemas/", ".xsd").iterator();
        } catch (IOException e) {
            throw new VerinecException("Could not read schemas in classpath",e );
        }
        if (! iter.hasNext())
	        logger.warning("Did not find any schemas. Will only work with network access.");
        String BASE_ADRESS = "http://diuf.unifr.ch/tns/projects/verinec/";	
        while(iter.hasNext()) {
            URL u = (URL) iter.next();
            String uri = urlEncode(u.toString());
            String schemaname = uri.substring(uri.lastIndexOf("/")+1, uri.length()-4); //it is the forward slash even on windows
            String namespace = BASE_ADDRESS + schemaname;
            result += namespace + " "+ uri+" ";
        }

	    logger.info("Using the following local schemas: "+result);
        return result;
    }
    
    /** As long as <a href="http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6274990">this bug in Java</a>
     * is not fixed, the jars must be in a path with only us-ascii letters and numbers.
     * 
     * If there are other characters in the path, the validation will require network access.
     * 
     * @param s An URL
     * @return The encoded URL. Currently just replaces space by %20.
     */
    public String urlEncode(String s) {
        return s.replaceAll(" ", "%20");
    }
    
}

